]> de.git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - cl_demo.c
patch from div0 that enables printf format warnings in gcc
[xonotic/darkplaces.git] / cl_demo.c
index 546ce450a2d10e267f61d4f66416b0aafcb375a4..3e7f1c50aadb579c3e624fd8eb1b94b3676a112a 100644 (file)
--- a/cl_demo.c
+++ b/cl_demo.c
@@ -20,6 +20,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "quakedef.h"
 
+int old_vsync = 0;
+
 void CL_FinishTimeDemo (void);
 
 /*
@@ -44,7 +46,7 @@ Called to play the next demo in the demo loop
 */
 void CL_NextDemo (void)
 {
-       char    str[1024];
+       char    str[MAX_INPUTLINE];
 
        if (cls.demonum == -1)
                return;         // don't play demos
@@ -54,7 +56,7 @@ void CL_NextDemo (void)
                cls.demonum = 0;
                if (!cls.demos[cls.demonum][0])
                {
-                       Con_Print("No demos listed with startdemos\n");
+                       Con_Print("No demos listed with startdemos\n");
                        cls.demonum = -1;
                        return;
                }
@@ -84,6 +86,10 @@ void CL_StopPlayback (void)
 
        if (cls.timedemo)
                CL_FinishTimeDemo ();
+
+       if (COM_CheckParm("-demo") || COM_CheckParm("-demolooponly"))
+               Host_Quit_f();
+
 }
 
 /*
@@ -110,7 +116,6 @@ void CL_WriteDemoMessage (void)
                FS_Write (cls.demofile, &f, 4);
        }
        FS_Write (cls.demofile, net_message.data, net_message.cursize);
-       FS_Flush (cls.demofile);
 }
 
 /*
@@ -157,6 +162,13 @@ void CL_ReadDemoMessage(void)
                                // count against the final report
                                if (host_framecount == cls.td_startframe)
                                        cls.td_starttime = realtime;
+                               if (host_framecount > cls.td_startframe + 2)
+                               {
+                                       cls.td_minframetime = min(cls.td_minframetime, cl.realframetime);
+                                       cls.td_maxframetime = max(cls.td_maxframetime, cl.realframetime);
+                               }
+                               else
+                                       cls.td_minframetime = cls.td_maxframetime = cl.realframetime;
                        }
                        else if (cl.time <= cl.mtime[0])
                        {
@@ -168,19 +180,23 @@ void CL_ReadDemoMessage(void)
                // get the next message
                FS_Read(cls.demofile, &net_message.cursize, 4);
                net_message.cursize = LittleLong(net_message.cursize);
+               if (net_message.cursize > net_message.maxsize)
+                       Host_Error("Demo message (%i) > net_message.maxsize (%i)", net_message.cursize, net_message.maxsize);
                VectorCopy(cl.mviewangles[0], cl.mviewangles[1]);
                for (i = 0;i < 3;i++)
                {
-                       r = FS_Read(cls.demofile, &f, 4);
+                       r = (int)FS_Read(cls.demofile, &f, 4);
                        cl.mviewangles[0][i] = LittleFloat(f);
                }
 
-               if (net_message.cursize > NET_MAXMESSAGE)
-                       Host_Error("Demo message > NET_MAXMESSAGE");
-               if (FS_Read(cls.demofile, net_message.data, net_message.cursize) == (size_t)net_message.cursize)
+               if (FS_Read(cls.demofile, net_message.data, net_message.cursize) == net_message.cursize)
                {
                        MSG_BeginReading();
                        CL_ParseServerMessage();
+
+                       // In case the demo contains a "svc_disconnect" message
+                       if (!cls.demoplayback)
+                               return;
                }
                else
                {
@@ -200,12 +216,9 @@ stop recording a demo
 */
 void CL_Stop_f (void)
 {
-       if (cmd_source != src_command)
-               return;
-
        if (!cls.demorecording)
        {
-               Con_Print("Not recording a demo.\n");
+               Con_Print("Not recording a demo.\n");
                return;
        }
 
@@ -218,7 +231,7 @@ void CL_Stop_f (void)
        FS_Close (cls.demofile);
        cls.demofile = NULL;
        cls.demorecording = false;
-       Con_Print("Completed demo\n");
+       Con_Print("Completed demo\n");
 }
 
 /*
@@ -233,25 +246,22 @@ void CL_Record_f (void)
        int c, track;
        char name[MAX_OSPATH];
 
-       if (cmd_source != src_command)
-               return;
-
        c = Cmd_Argc();
        if (c != 2 && c != 3 && c != 4)
        {
-               Con_Print("record <demoname> [<map> [cd track]]\n");
+               Con_Print("record <demoname> [<map> [cd track]]\n");
                return;
        }
 
        if (strstr(Cmd_Argv(1), ".."))
        {
-               Con_Print("Relative pathnames are not allowed.\n");
+               Con_Print("Relative pathnames are not allowed.\n");
                return;
        }
 
        if (c == 2 && cls.state == ca_connected)
        {
-               Con_Printf("Can not record - already connected to server\nClient demo recording must be started before connecting\n");
+               Con_Print("Can not record - already connected to server\nClient demo recording must be started before connecting\n");
                return;
        }
 
@@ -259,7 +269,7 @@ void CL_Record_f (void)
        if (c == 4)
        {
                track = atoi(Cmd_Argv(3));
-               Con_Printf ("Forcing CD track to %i\n", cls.forcetrack);
+               Con_Printf("Forcing CD track to %i\n", cls.forcetrack);
        }
        else
                track = -1;
@@ -273,16 +283,16 @@ void CL_Record_f (void)
                Cmd_ExecuteString ( va("map %s", Cmd_Argv(2)), src_command);
 
        // open the demo file
-       Con_Printf ("recording to %s.\n", name);
-       cls.demofile = FS_Open (name, "wb", false);
+       Con_Printf("recording to %s.\n", name);
+       cls.demofile = FS_Open (name, "wb", false, false);
        if (!cls.demofile)
        {
-               Con_Print("ERROR: couldn't open.\n");
+               Con_Print("ERROR: couldn't open.\n");
                return;
        }
 
        cls.forcetrack = track;
-       FS_Printf (cls.demofile, "%i\n", cls.forcetrack);
+       FS_Printf(cls.demofile, "%i\n", cls.forcetrack);
 
        cls.demorecording = true;
 }
@@ -297,40 +307,38 @@ play [demoname]
 */
 void CL_PlayDemo_f (void)
 {
-       char    name[256];
+       char    name[MAX_QPATH];
        int c;
        qboolean neg = false;
 
-       if (cmd_source != src_command)
-               return;
-
        if (Cmd_Argc() != 2)
        {
-               Con_Print("play <demoname> : plays a demo\n");
+               Con_Print("play <demoname> : plays a demo\n");
                return;
        }
 
        // disconnect from server
        CL_Disconnect ();
+       Host_ShutdownServer ();
 
        // update networking ports (this is mainly just needed at startup)
-       NetConn_ClientFrame();
+       NetConn_UpdateSockets();
 
        // open the demo file
        strlcpy (name, Cmd_Argv(1), sizeof (name));
        FS_DefaultExtension (name, ".dem", sizeof (name));
+       cls.protocol = PROTOCOL_QUAKE;
 
-       Con_Printf ("Playing demo from %s.\n", name);
-       cls.demofile = FS_Open (name, "rb", false);
+       Con_Printf("Playing demo from %s.\n", name);
+       cls.demofile = FS_Open (name, "rb", false, false);
        if (!cls.demofile)
        {
-               Con_Print("ERROR: couldn't open.\n");
+               Con_Print("ERROR: couldn't open.\n");
                cls.demonum = -1;               // stop demo loop
                return;
        }
 
-       SCR_BeginLoadingPlaque ();
-
+       strlcpy(cls.demoname, name, sizeof(cls.demoname));
        cls.demoplayback = true;
        cls.state = ca_connected;
        cls.forcetrack = 0;
@@ -353,18 +361,23 @@ CL_FinishTimeDemo
 */
 void CL_FinishTimeDemo (void)
 {
-       int             frames;
-       double  time; // LordHavoc: changed timedemo accuracy to double
+       int frames;
+       double time; // LordHavoc: changed timedemo accuracy to double
+       double fpsmin, fpsavg, fpsmax; // report min/avg/max fps
 
        cls.timedemo = false;
 
 // the first frame didn't count
        frames = (host_framecount - cls.td_startframe) - 1;
        time = realtime - cls.td_starttime;
-       if (!time)
-               time = 1;
-       // LordHavoc: timedemo now prints out 7 digits of fraction
-       Con_Printf ("%i frames %5.7f seconds %5.7f fps\n", frames, time, frames/time);
+       fpsmin = cls.td_maxframetime > 0 ? 1.0 / cls.td_maxframetime : 0;
+       fpsavg = time > 0 ? frames / time : 0;
+       fpsmax = cls.td_minframetime > 0 ? 1.0 / cls.td_minframetime : 0;
+       // LordHavoc: timedemo now prints out 7 digits of fraction, and min/avg/max
+       Con_Printf("%i frames %5.7f seconds %5.7f fps\nmin/avg/max: %5.7f/%5.7f/%5.7f\n", frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
+       Log_Printf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | result %i frames %5.7f seconds %5.7f fps min/avg/max: %5.7f/%5.7f/%5.7f\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, frames, time, fpsavg, fpsmin, fpsavg, fpsmax);
+       if (COM_CheckParm("-benchmark"))
+               Host_Quit_f();
 }
 
 /*
@@ -376,12 +389,9 @@ timedemo [demoname]
 */
 void CL_TimeDemo_f (void)
 {
-       if (cmd_source != src_command)
-               return;
-
        if (Cmd_Argc() != 2)
        {
-               Con_Print("timedemo <demoname> : gets demo speeds\n");
+               Con_Print("timedemo <demoname> : gets demo speeds\n");
                return;
        }
 
@@ -393,7 +403,6 @@ void CL_TimeDemo_f (void)
        // instantly hide console and deactivate it
        key_dest = key_game;
        key_consoleactive = 0;
-       scr_conlines = 0;
        scr_con_current = 0;
 
        cls.timedemo = true;